package guru.kake.ronove

import javafx.scene.control.TextField
import javafx.scene.text.FontSmoothingType
import kotlinx.html.*
import kotlinx.html.stream.createHTML
import netscape.javascript.JSObject
import tornadofx.*

/**
 * Tab class to display text using HTML formatting
 */
class ReaderTab : View("Reader") {
    private val cbobject: WebViewCallback = WebViewCallback()
    override val root = vbox {
        webview {
            style {
                fontSmoothingType = FontSmoothingType.GRAY
            }
            engine.loadContent(createHTML().html {
                head {
                    style {
                        unsafe { raw("body { font-size: 48px; }") }
                    }
                }
                body {
                    onLoad = "alert()"
                    p {
                        ruby { +"私"; rt { +"わたし" }}
                        +"の"
                        ruby { +"猫"; rt { +"ねこ" }}
                        +"わかわいいです"
                        ruby("a") {+"b"}
                        a("#") {
                            onClick = "cb.test('foo');"
                            +"linkki"
                        }
                    }

                }
            })

            engine.setOnAlert {
                (engine.executeScript("window") as JSObject).setMember("cb", cbobject)
            }
        }
    }
}

/**
 * Class for receiving events from JS in [ReaderTab]
 */
@Suppress("unused") // IDEA doesn't detect uses of these functions since they are called from JS in the WebView
class WebViewCallback {
    fun test(message: String) {
        println(message)
    }
}

/**
 * Tab class for displaying dictionaries
 */
class DictionaryTab : View("Dictionary") {
    override val root = vbox {

        val xmlParser = XMLParser()
        val ret = xmlParser.parse("C:\\repos\\kanjisho\\kanjidic2.xml")
        val tree = BST<Character>(ret)

        //var x = tree.find(Character("猫"))
        hbox {
            val search = textfield()
            button("Search") {
                action {
                    val c = tree.find(Character(search.text))
                    if (c != null) {
                        var kun: String = c.rmgroups[0].reading.find { it.first == "ja_kun" }?.second ?: ""
                        var on: String = c.rmgroups[0].reading.find { it.first == "ja_on" }?.second ?: ""
                        var meaning: String = c.rmgroups[0].meaning.find { it.first == null }?.second ?: ""

                        this@vbox += DictionaryEntry(c.literal, kun, on, meaning)
                    }
                }
            }
        }
    }
}

class DictionaryEntry(
    private val kanji: String,
    private val kun: String,
    private val on: String,
    private val meaning: String
) : Fragment("") {
    override val root = hbox {
        label(kanji) {
            addClass(RonoveStylesheet.bigLabel)
        }
        vbox {
            label("Kun: $kun")
            label("On: $on")
            label(meaning)
        }
    }
}

/**
 * Tab class for displaying original material in image form
 */
class ImageTab : View("Image") {
    override val root = vbox {
        imageview("/yopage.jpg") {
            fitWidth = 300.0
            isPreserveRatio = true
            isSmooth = true
        }
    }
}

/**
 * Tab class for a separate window letting the user run arbitrary strings through machine translation
 */
class TranslatorTab : View("Machine translation") {
    private var fromField: TextField by singleAssign()
    private var toField: TextField by singleAssign()

    //TODO allow switching languages on the fly
    override val root = vbox {
        vbox {
            label("Japanese")
            fromField = textfield()
            button("To English") {
                action {
                    TranslationBackend.destinationLanguage = "en-US"
                    TranslationBackend.sourceLanguage = "ja"
                    toField.text = TranslationBackend(fromField.text)
                }
            }
        }
        vbox {
            label("English")
            toField = textfield()
            button("To Japanese") {
                action {
                    TranslationBackend.destinationLanguage = "ja"
                    TranslationBackend.sourceLanguage = "en"
                    fromField.text = TranslationBackend(toField.text)
                }
            }
        }
    }

}