package guru.kake.ronove

import java.io.FileInputStream
import javax.xml.namespace.QName
import javax.xml.stream.XMLEventReader
import javax.xml.stream.XMLInputFactory
import javax.xml.stream.events.XMLEvent


class Character(var literal: String = "") : Comparable<Character> {
    var rmgroups: MutableList<RMGroup> = mutableListOf()

    fun print() {
        println("=== Character: $literal ===")
        for (n in rmgroups) {
            n.print()
        }
    }

    override fun equals(other: Any?): Boolean {
        if (other is Character) {
            return this.literal == other.literal
        }
        return super.equals(other)
    }
    override fun compareTo(other: Character) = this.literal.compareTo(other.literal)
}

class RMGroup {
    var reading: MutableList<Pair<String?, String>> = mutableListOf()
    var meaning: MutableList<Pair<String?, String>> = mutableListOf()

    fun print() {
        println("-- RMGroup --")
        for  (r in reading) {
            println("Reading: ${r.second} (${r.first})")
        }
        for  (m in meaning) {
            println("Meaning: ${m.second} (${m.first})")
        }
    }
}

class XMLParser {
    fun parse(path: String): List<Character> {
        val parsed: MutableList<Character> = mutableListOf()
        var character: Character? = null
        var rmgroup: RMGroup? = null

        val xmlInputFactory: XMLInputFactory = XMLInputFactory.newInstance()
        val reader: XMLEventReader = xmlInputFactory.createXMLEventReader(FileInputStream(path))

        while (reader.hasNext()) {
            val nextEvent: XMLEvent = reader.nextEvent()
            when {
                nextEvent.isStartElement -> {
                    val startElement = nextEvent.asStartElement()
                    when (startElement.name.localPart.toString()) {
                        "character" -> character = Character()
                        "literal" -> {
                            character!!.literal = reader.nextEvent().asCharacters().data
                        }
                        "reading_meaning" -> rmgroup = RMGroup()
                        "reading" -> {
                            rmgroup!!.reading.add(Pair(
                                startElement.getAttributeByName(QName("r_type"))?.value,
                                reader.nextEvent().asCharacters().data
                            ))
                        }
                        "meaning" -> {
                            rmgroup!!.meaning.add(Pair(
                                startElement.getAttributeByName(QName("m_lang"))?.value,
                                reader.nextEvent().asCharacters().data
                            ))
                        }
                    }
                }
                nextEvent.isEndElement -> {
                    when (nextEvent.asEndElement().name.localPart.toString()) {
                        "character" -> {
                            parsed.add(character!!)
                            character = null
                        }
                        "reading_meaning" -> {
                            character!!.rmgroups.add(rmgroup!!)
                            rmgroup = null
                        }
                    }
                }
            }
        }
        return parsed.toList()
    }
}